﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace XamComboEditorErrorInfo
{
    public class MainWindowVM : IDataErrorInfo, INotifyPropertyChanged
    {
        int intID = 4;
        string strName;
        string strCompany;
        private Dictionary<string, List<string>> errors = new Dictionary<string, List<string>>();
        string _selectedItem;

        public int ID
        {
            get { return intID; }
            set
            {
                if (intID != value)
                {
                    intID = value;
                }
            }
        }

        public string Name
        {
            get { return strName; }
            set
            {
                if (strName != value)
                {
                    strName = value;
                }
            }
        }

        public string Company
        {
            get { return strCompany; }
            set
            {
                if (strCompany != value)
                {
                    strCompany = value;
                }
            }
        }



        public DataTable CompanyList { get; set; }
        public string SelectedItem
        {
            get { return _selectedItem; }
            set
            {
                if (this._selectedItem != value)
                {
                    if (value == null)
                        AddError("Selected1", "Selection 1 cannot be null", false);
                    else
                        RemoveError("Selected1", "Selection 1 cannot be null");

                    _selectedItem = value;
                    OnPropertyChanged("SelectedItem");
                }
            }
        }

        public MainWindowVM()
        {
            DataTable dt = new DataTable();

            dt.Columns.Add("ID");
            dt.Columns.Add("Name");

            for (int i = 1; i <= 10; i++)
            {
                dt.Rows.Add(i, "Company Name that is really, really, really long" + i);
            }

            CompanyList = dt;

        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        #region INotifyDataErrorInfo

        public void AddError(string propertyName, string error, bool isWarning)
        {
            System.Diagnostics.Debug.WriteLine(propertyName + ": " + error);
            if (!errors.ContainsKey(propertyName))
                errors[propertyName] = new List<string>();

            if (!errors[propertyName].Contains(error))
            {
                if (isWarning) errors[propertyName].Add(error);
                else errors[propertyName].Insert(0, error);
                RaiseErrorsChanged(propertyName);
            }
        }

        public void RemoveError(string propertyName, string error)
        {
            if (errors.ContainsKey(propertyName) &&
                errors[propertyName].Contains(error))
            {
                errors[propertyName].Remove(error);
                if (errors[propertyName].Count == 0) errors.Remove(propertyName);
                RaiseErrorsChanged(propertyName);
            }
        }

        public void RaiseErrorsChanged(string propertyName)
        {
            if (ErrorsChanged != null)
                ErrorsChanged(this, new DataErrorsChangedEventArgs(propertyName));
        }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public IEnumerable GetErrors(string propertyName)
        {
            if (errors.ContainsKey(propertyName))
                return errors[propertyName];
            return null;
        }

        public bool HasErrors
        {
            get { return errors.Count > 0; }
        }

        #region " Validation "

        public string Error
        {
            get { return ""; }
        }

        public string this[string columnName]
        {

            get
            {
                string sError = string.Empty;
                string sDisplayName = string.Empty;

                switch (columnName)
                {
                    case "Company":
                        sDisplayName = "Company";
                        if (string.IsNullOrEmpty(Company) || columnName == "1")
                        {
                            sError = "Required";
                        }
                        break;
                    case "Name":
                        sDisplayName = "Name";
                        if (string.IsNullOrEmpty(Name))
                        {
                            sError = "Required";
                        }

                        break;

                }

                //Update the validation list
                // base.ChangeValidation(columnName, sDisplayName, sError);

                return sError;

            }
        }

        #endregion

        #endregion




    }
}
