﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace XamComboEditorErrorInfo
{
    public class ViewModel : INotifyPropertyChanged, INotifyDataErrorInfo
    {
        private string _sel1;
        private string _sel2;
        private string _sel3;
        private Dictionary<string, List<string>> errors = new Dictionary<string, List<string>>();

        public ObservableCollection<ComboItem> ComboList { get; set; }

        public ViewModel()
        {
            ComboList = new ObservableCollection<ComboItem>();
            for (int i = 0; i < 10; i++)
            {
                ComboList.Add(new ComboItem() { ID = i, Name = "Item" + i.ToString() });
            }

            Selected1 = null;
            Selected2 = null;
            Selected3 = null;

            AddError("Selected1", "Selection 1 cannot be null", false);
            AddError("Selected2", "Selection 2 cannot be null", false);
            AddError("Selected3", "Selection 3 cannot be null", false);

          
        }


        public string Selected1
        {
            get { return _sel1; }
            set
            {
                 if (this._sel1 != value )
                {
                    if (value == null)
                        AddError("Selected1", "Selection 1 cannot be null", false);
                    else
                        RemoveError("Selected1", "Selection 1 cannot be null");

                    _sel1 = value;
                    OnPropertyChanged("Selected1");
                }
            }
            
        }
        public string Selected3
        {
            get { return _sel3; }
            set
            {
                if (this._sel3 != value)
                {
                    if (value == null)
                        AddError("Selected3", "Selection 3 cannot be null", false);
                    else
                        RemoveError("Selected3", "Selection 3 cannot be null");

                    _sel3 = value;
                    OnPropertyChanged("Selected3");
                   
                }
            }

        }
        public string Selected2
        {
            get { return _sel2; }
            set
            {
                if (this._sel2 != value)
                {
                    if (value == null)
                        AddError("Selected2", "Selection 2 cannot be null", false);
                    else
                        RemoveError("Selected2", "Selection 2 cannot be null");

                    _sel2 = value;
                    OnPropertyChanged("Selected2");
                }
            }

        }


        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        #region INotifyDataErrorInfo

        public void AddError(string propertyName, string error, bool isWarning)
        {
            System.Diagnostics.Debug.WriteLine(propertyName + ": " + error);
            if (!errors.ContainsKey(propertyName))
                errors[propertyName] = new List<string>();

            if (!errors[propertyName].Contains(error))
            {
                if (isWarning) errors[propertyName].Add(error);
                else errors[propertyName].Insert(0, error);
                RaiseErrorsChanged(propertyName);
            }
        }

        public void RemoveError(string propertyName, string error)
        {
            if (errors.ContainsKey(propertyName) &&
                errors[propertyName].Contains(error))
            {
                errors[propertyName].Remove(error);
                if (errors[propertyName].Count == 0) errors.Remove(propertyName);
                RaiseErrorsChanged(propertyName);
            }
        }

        public void RaiseErrorsChanged(string propertyName)
        {
            if (ErrorsChanged != null)
                ErrorsChanged(this, new DataErrorsChangedEventArgs(propertyName));
        }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public IEnumerable GetErrors(string propertyName)
        {
            if (errors.ContainsKey(propertyName))
                return errors[propertyName];
            return null;
        }

        public bool HasErrors
        {
            get { return errors.Count > 0; }
        }

        #endregion
    }

    public class ComboItem : INotifyPropertyChanged
    {
        private int _id;
        private string _name;


        public int ID
        {
            get { return _id;}
            set
            {
                if (_id != value)
                {
                    _id = value;
                    OnPropertyChanged("ID");
                }
            }
        }
        public string Name
        {
            get { return _name; }
            set
            {
                if (_name != value)
                {
                    _name = value;
                    OnPropertyChanged("Name");
                }
            }
        }
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }
}
